@@ -27,19 +27,20 @@ from typing import Optional, List, Dict, Sequence, Set, TYPE_CHECKING
 import enum
 import copy
 
-from PyQt5.QtCore import Qt
-from PyQt5.QtGui import QStandardItemModel, QStandardItem, QFont
-from PyQt5.QtWidgets import QAbstractItemView, QMenu, QLabel, QHBoxLayout
+from PyQt6.QtCore import Qt
+from PyQt6.QtGui import QStandardItemModel, QStandardItem, QFont
+from PyQt6.QtWidgets import QAbstractItemView, QMenu
 
 from electrum.i18n import _
 from electrum.bitcoin import is_address
 from electrum.transaction import PartialTxInput, PartialTxOutput
-from electrum.lnutil import LN_MAX_FUNDING_SAT, MIN_FUNDING_SAT
+from electrum.lnutil import MIN_FUNDING_SAT
 from electrum.util import profiler
 
-from .util import ColorScheme, MONOSPACE_FONT, EnterButton
-from .my_treeview import MyTreeView
+from .util import ColorScheme, MONOSPACE_FONT
+from .my_treeview import MyTreeView, MySortModel
 from .new_channel_dialog import NewChannelDialog
+from ..messages import MSG_FREEZE_ADDRESS, MSG_FREEZE_COIN
 
 if TYPE_CHECKING:
     from .main_window import ElectrumWindow
@@ -66,7 +67,8 @@ class UTXOList(MyTreeView):
     filter_columns = [Columns.ADDRESS, Columns.LABEL, Columns.OUTPOINT]
     stretch_column = Columns.LABEL
 
-    ROLE_PREVOUT_STR = Qt.UserRole + 1000
+    ROLE_PREVOUT_STR = Qt.ItemDataRole.UserRole + 1000
+    ROLE_SORT_ORDER = Qt.ItemDataRole.UserRole + 1001
     key_role = ROLE_PREVOUT_STR
 
     def __init__(self, main_window: 'ElectrumWindow'):
@@ -78,42 +80,56 @@ class UTXOList(MyTreeView):
         self._utxo_dict = {}
         self.wallet = self.main_window.wallet
         self.std_model = QStandardItemModel(self)
-        self.setModel(self.std_model)
-        self.setSelectionMode(QAbstractItemView.ExtendedSelection)
+        self.proxy = MySortModel(self, sort_role=self.ROLE_SORT_ORDER)
+        self.proxy.setSourceModel(self.std_model)
+        self.setModel(self.proxy)
+        self.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
         self.setSortingEnabled(True)
 
     def create_toolbar(self, config):
         toolbar, menu = self.create_toolbar_with_menu('')
         self.num_coins_label = toolbar.itemAt(0).widget()
         menu.addAction(_('Coin control'), lambda: self.add_selection_to_coincontrol())
+
+        def cb():
+            self.main_window.utxo_list.refresh_all()  # for coin frozen status
+            self.main_window.update_status()  # frozen balance
+        menu.addConfig(config.cv.WALLET_FREEZE_REUSED_ADDRESS_UTXOS, callback=cb)
         return toolbar
 
     @profiler(min_threshold=0.05)
     def update(self):
         # not calling maybe_defer_update() as it interferes with coincontrol status bar
+        self.proxy.setDynamicSortFilter(False)  # temp. disable re-sorting after every change
         utxos = self.wallet.get_utxos()
-        utxos.sort(key=lambda x: x.block_height, reverse=True)
         self._maybe_reset_coincontrol(utxos)
         self._utxo_dict = {}
-        self.model().clear()
+        self.std_model.clear()
         self.update_headers(self.__class__.headers)
         for idx, utxo in enumerate(utxos):
             name = utxo.prevout.to_str()
             self._utxo_dict[name] = utxo
             labels = [""] * len(self.Columns)
+            amount_str = self.main_window.format_amount(
+                utxo.value_sats(), whitespaces=True)
+            amount_str_nots = self.main_window.format_amount(
+                utxo.value_sats(), whitespaces=False, add_thousands_sep=False)
             labels[self.Columns.OUTPOINT] = str(utxo.short_id)
             labels[self.Columns.ADDRESS] = utxo.address
-            labels[self.Columns.AMOUNT] = self.main_window.format_amount(utxo.value_sats(), whitespaces=True)
+            labels[self.Columns.AMOUNT] = amount_str
             utxo_item = [QStandardItem(x) for x in labels]
             self.set_editability(utxo_item)
             utxo_item[self.Columns.OUTPOINT].setData(name, self.ROLE_PREVOUT_STR)
+            utxo_item[self.Columns.AMOUNT].setData(amount_str_nots, self.ROLE_CLIPBOARD_DATA)
             utxo_item[self.Columns.ADDRESS].setFont(QFont(MONOSPACE_FONT))
             utxo_item[self.Columns.AMOUNT].setFont(QFont(MONOSPACE_FONT))
             utxo_item[self.Columns.PARENTS].setFont(QFont(MONOSPACE_FONT))
             utxo_item[self.Columns.OUTPOINT].setFont(QFont(MONOSPACE_FONT))
-            self.model().insertRow(idx, utxo_item)
+            self.std_model.insertRow(idx, utxo_item)
             self.refresh_row(name, idx)
         self.filter()
+        self.proxy.setDynamicSortFilter(True)
+        self.sortByColumn(self.Columns.OUTPOINT, Qt.SortOrder.DescendingOrder)
         self.update_coincontrol_bar()
         self.num_coins_label.setText(_('{} unspent transaction outputs').format(len(utxos)))
 
@@ -134,10 +150,15 @@ class UTXOList(MyTreeView):
         utxo = self._utxo_dict[key]
         utxo_item = [self.std_model.item(row, col) for col in self.Columns]
         txid = utxo.prevout.txid.hex()
-        parents = self.wallet.get_tx_parents(txid)
-        utxo_item[self.Columns.PARENTS].setText('%6s'%len(parents))
+        num_parents = self.wallet.get_num_parents(txid)
+        utxo_item[self.Columns.PARENTS].setText('%6s'%num_parents if num_parents else '-')
         label = self.wallet.get_label_for_txid(txid) or ''
         utxo_item[self.Columns.LABEL].setText(label)
+        sort_key = (
+            self.wallet.adb.tx_height_to_sort_height(utxo.block_height),  # sort by block height
+            str(utxo.short_id),                                           # order inside block (if mined), or just txid
+        )
+        utxo_item[self.Columns.OUTPOINT].setData(sort_key, self.ROLE_SORT_ORDER)
         SELECTED_TO_SPEND_TOOLTIP = _('Coin selected to be spent')
         if key in self._spend_set:
             tooltip = key + "\n" + SELECTED_TO_SPEND_TOOLTIP
@@ -221,7 +242,10 @@ class UTXOList(MyTreeView):
             return False
         value = sum(x.value_sats() for x in coins)
         min_amount = self.wallet.lnworker.swap_manager.get_min_amount()
-        max_amount = self.wallet.lnworker.swap_manager.max_amount_forward_swap()
+        max_amount = self.wallet.lnworker.swap_manager.client_max_amount_forward_swap()
+        if min_amount is None or max_amount is None:
+            # we need to fetch data from swap server
+            return True
         if value < min_amount:
             return False
         if max_amount is None or value > max_amount:
@@ -238,7 +262,7 @@ class UTXOList(MyTreeView):
         if self.wallet.lnworker is None:
             return False
         value = sum(x.value_sats() for x in coins)
-        return value >= MIN_FUNDING_SAT and value <= LN_MAX_FUNDING_SAT
+        return value >= MIN_FUNDING_SAT and value <= self.config.LIGHTNING_MAX_FUNDING_SAT
 
     def open_channel_with_coins(self, coins):
         # todo : use a single dialog in new flow
@@ -259,6 +283,9 @@ class UTXOList(MyTreeView):
         return is_address(text)
 
     def pay_to_clipboard_address(self, coins):
+        if not self.clipboard_contains_address():
+            self.main_window.show_error(_('Clipboard doesn\'t contain a valid address'))
+            return
         addr = self.main_window.app.clipboard().text()
         outputs = [PartialTxOutput.from_address_and_value(addr, '!')]
         #self.clear_coincontrol()
@@ -309,30 +336,38 @@ class UTXOList(MyTreeView):
             utxo = coins[0]
             addr = utxo.address
             menu_freeze = menu.addMenu(_("Freeze"))
+            menu_freeze.setToolTipsVisible(True)
             if not self.wallet.is_frozen_coin(utxo):
-                menu_freeze.addAction(_("Freeze Coin"), lambda: self.main_window.set_frozen_state_of_coins([utxo], True))
+                act = menu_freeze.addAction(_("Freeze Coin"), lambda: self.main_window.set_frozen_state_of_coins([utxo], True))
             else:
-                menu_freeze.addAction(_("Unfreeze Coin"), lambda: self.main_window.set_frozen_state_of_coins([utxo], False))
+                act = menu_freeze.addAction(_("Unfreeze Coin"), lambda: self.main_window.set_frozen_state_of_coins([utxo], False))
+            act.setToolTip(MSG_FREEZE_COIN)
             if not self.wallet.is_frozen_address(addr):
-                menu_freeze.addAction(_("Freeze Address"), lambda: self.main_window.set_frozen_state_of_addresses([addr], True))
+                act = menu_freeze.addAction(_("Freeze Address"), lambda: self.main_window.set_frozen_state_of_addresses([addr], True))
             else:
-                menu_freeze.addAction(_("Unfreeze Address"), lambda: self.main_window.set_frozen_state_of_addresses([addr], False))
+                act = menu_freeze.addAction(_("Unfreeze Address"), lambda: self.main_window.set_frozen_state_of_addresses([addr], False))
+            act.setToolTip(MSG_FREEZE_ADDRESS)
         elif len(coins) > 1:  # multiple items selected
             menu.addSeparator()
             addrs = [utxo.address for utxo in coins]
             is_coin_frozen = [self.wallet.is_frozen_coin(utxo) for utxo in coins]
             is_addr_frozen = [self.wallet.is_frozen_address(utxo.address) for utxo in coins]
             menu_freeze = menu.addMenu(_("Freeze"))
+            menu_freeze.setToolTipsVisible(True)
             if not all(is_coin_frozen):
-                menu_freeze.addAction(_("Freeze Coins"), lambda: self.main_window.set_frozen_state_of_coins(coins, True))
+                act = menu_freeze.addAction(_("Freeze Coins"), lambda: self.main_window.set_frozen_state_of_coins(coins, True))
+                act.setToolTip(MSG_FREEZE_COIN)
             if any(is_coin_frozen):
-                menu_freeze.addAction(_("Unfreeze Coins"), lambda: self.main_window.set_frozen_state_of_coins(coins, False))
+                act = menu_freeze.addAction(_("Unfreeze Coins"), lambda: self.main_window.set_frozen_state_of_coins(coins, False))
+                act.setToolTip(MSG_FREEZE_COIN)
             if not all(is_addr_frozen):
-                menu_freeze.addAction(_("Freeze Addresses"), lambda: self.main_window.set_frozen_state_of_addresses(addrs, True))
+                act = menu_freeze.addAction(_("Freeze Addresses"), lambda: self.main_window.set_frozen_state_of_addresses(addrs, True))
+                act.setToolTip(MSG_FREEZE_ADDRESS)
             if any(is_addr_frozen):
-                menu_freeze.addAction(_("Unfreeze Addresses"), lambda: self.main_window.set_frozen_state_of_addresses(addrs, False))
+                act = menu_freeze.addAction(_("Unfreeze Addresses"), lambda: self.main_window.set_frozen_state_of_addresses(addrs, False))
+                act.setToolTip(MSG_FREEZE_ADDRESS)
 
-        menu.exec_(self.viewport().mapToGlobal(position))
+        menu.exec(self.viewport().mapToGlobal(position))
 
     def get_filter_data_from_coordinate(self, row, col):
         if col == self.Columns.OUTPOINT:
